
from nbtlib.tag import Int, Long, Double, String, List, Compound, IntArray

from .util import RequiredKeyMissingException



class BlockEntity:

    def __init__(self, str_or_nbt):

        if isinstance(str_or_nbt, str):
            self._data = Compound({'id': String(str_or_nbt)})
        else:
            self._data = str_or_nbt

        keys = self._data.keys()
        if 'id' not in keys:
            raise RequiredKeyMissingException('id')
        if 'x' not in keys:
            self._data['x'] = Int(0)
        if 'y' not in keys:
            self._data['y'] = Int(0)
        if 'z' not in keys:
            self._data['z'] = Int(0)

        self._id = self._data['id']
        self._position = tuple([int(self._data[coord]) for coord in ['x', 'y', 'z']])

    def _tonbt(self):
        return self._data

    @staticmethod
    def fromnbt(nbt):
        return BlockEntity(nbt)

    def add_tag(self, key, tag):
        self._data[key] = tag
        if key == 'id':
            self._id = str(tag)

        pos = self._position
        if key == 'x':
            self._position = (int(tag), pos[1], pos[2])
        if key == 'y':
            self._position = (pos[0], int(tag), pos[2])
        if key == 'z':
            self._position = (pos[0], pos[1], int(tag))

    def get_tag(self, key):
        try:
            return self._data[key]
        except KeyError:
            raise

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, data):
        self._data = BlockEntity(data).data
        self._id = str(self._data['id'])
        self._position = tuple([int(self._data[coord]) for coord in ['x', 'y', 'z']])

    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, id):
        self._id = id
        self._data['id'] = String(self._id)

    @property
    def position(self):
        return self._position

    @position.setter
    def position(self, position):
        self._position = position
        for coord, index in [('x', 0), ('y', 1), ('z', 2)]:
            self._data[coord] = Int(self._position[index])