




import json

import numpy as np
from nbtlib.tag import Int, Long, Double, String, List, Compound, IntArray


from .Block import Block


from .info import *
from .storage import LitematicaBitArray, DiscriminatingDictionary








class BlockState:

    def __init__(self, block:Block, properties={}):

        self.__block = block
        self.__properties = DiscriminatingDictionary(self.__validate, properties)

    @property
    def blockid(self):
        return self.__block.getid()

    def _tonbt(self):
        root = Compound()
        root["Name"] = String(self.blockid)
        properties = {String(k): String(v) for k, v in self.__properties.items()}
        if len(properties) > 0:
            root["Properties"] = Compound(properties)
        return root

    @staticmethod
    def fromnbt(nbt):
        bid = str(nbt["Name"])
        if "Properties" in nbt:
            properties = {str(k): str(v) for k, v in nbt["Properties"].items()}
        else:
            properties = {}
        blockstate = BlockState(bid, properties=properties)
        return blockstate



    def __validate(self, k, v):
        if type(k) is not str or type(v) is not str:
            return False, "Blockstate properties should be a string => string dictionary"
        return True, ""

    def __eq__(self, other):
        if not isinstance(other, BlockState):
            raise ValueError("Can only compare blockstates with blockstates")
        return other.blockid == self.blockid and other.__properties == self.__properties

    def __repr__(self):
        return self.blockid + json.dumps(self.__properties)

    def __getitem__(self, key):
        return self.__properties[key]

    def __len__(self):
        return len(self.__properties)